/* Basic definitions */



// verify if has been declared, if no do it
#ifndef __3DFC_DEFS_H__



// signals file inclusion
#define __3DFC_DEFS_H__



// define compilation platform target and
// other compilation features
#include "../config.h"



// version
#define _3DFC_VERSION "beta 8"



// COMPONENT SYSTEM


// possible component types:

// just to clean the slot
#define TYPE_LET_SLOT_EMPTY 0

// unknow the component type
#define TYPE_UNKNOWN 1

// control the object position and orientation with keyboard
#define TYPE_KBCONTROL 2

// general data storage component
#define TYPE_DATA 3

// the reference system component (CCameraSystem)
// (has the position and orientation of the object)
#define TYPE_CAMSYS 4

// has a poly mesh (CMesh)
#define TYPE_MESH 5

// has a group, set, of sectors
#define TYPE_SECTORSSET 6

// a omni (spherical) light source
#define TYPE_LIGHT_SOURCE_OMNI 7

// has a handler (pointer) to one external C function called
// when component is activated
#define TYPE_FUNCTION_HANDLER 8

// can move object, controling many ref sys parameters
#define TYPE_MOVE 9



// object slots/messages:

// slot not used
#define SLOT_UNKNOWN 0

// has the move control with keyboard input
#define SLOT_REFSYS_KB_CTRL 1

// the external description of the object
#define SLOT_BODY 2

// the position and orientation of the object
#define SLOT_REFSYS 3

// the internal description of the object (set of sectors or other thing)
#define SLOT_INTERIOR 4

// the light source
#define SLOT_LIGHT_SOURCE 5

// slots reserved for user functions (mainly CComponentFunctionHandler)
#define SLOT_FOR_USER_0 6
#define SLOT_FOR_USER_1 7
#define SLOT_FOR_USER_2 8
#define SLOT_FOR_USER_3 9
#define SLOT_FOR_USER_4 10
#define SLOT_FOR_USER_5 11
#define SLOT_FOR_USER_6 12
#define SLOT_FOR_USER_7 13
#define SLOT_FOR_USER_8 14
#define SLOT_FOR_USER_9 15

// slot with motion capabilities
#define SLOT_MOVE 16

// casts
#define CAST_NONE 0
#define CAST_BYTE 1
#define CAST_DWORD 2
#define CAST_FLOAT 3
#define CAST_CMESH 4
#define CAST_CCAMERASYSTEM 5



// DATA TYPES

// unsigned int 8 bits
#define BYTE unsigned char

// int (signed and unsigned) 32 bits
#define INT  int
#define UINT unsigned int
#define DWORD unsigned int

// int (signed and unsigned) 16 bits
#define INT16  short int
#define UINT16 unsigned short int
#define WORD unsigned short int

// can receive a pointer or a value
// (usefull for portability to 64 bits systems - pointers have 64 bits)
#define PTRVAL unsigned int

// boolean
#define BOOL unsigned char
#define TRUE 1
#define FALSE 0



// CLASSES PREDECLARATIONS

class CFilePCX;
class CTexture;

class CFileTXE;
class CObject;
class CScenery;

class CNodeSector;
class CNodeObject;
class CNodeComponent;

class CMesh;

class CReferenceSystem;



// MAIN STRUCTURES

// describe a start and end value for an interval
struct tagInterval {
   INT
      nStart,
      nEnd;
};

// describe a location in 2D plane, coordenate has only 2 components: x and y
struct tagXY {
   INT
      nX,
      nY;
};

// describe a location in 3D space, coordenate has 3 components: x, y, z
struct tagXYZ {
   INT
      nX,
      nY,
      nZ;
};

// describe a location in 3D space, coordenate has 3 components: x, y, z
// this version use float
struct tagfXYZ {
   float
      fX,
      fY,
      fZ;
};

// define a rectangle
struct tagRectangle {
   INT nStartX;
   INT nStartY;
   INT nEndX;
   INT nEndY;
};

// define the destination of a portal polygon
struct tagPortal {
   BYTE yCode;             // 0 - not portal / 1 - portal to interior / 2 - portal to exterior
   DWORD dwSectorID;       // destin sector ID (used case code == 1)
   CNodeSector* poSector;  // pointer to destin sector (used case code == 1)
};

// data for one the message for one event that can be generated
struct tagMessage {

   CNodeObject* poObj;  // point the destin object
                        // If the pointer is NULL, the
                        // message is without destin.
   DWORD dwSlotID;      // Slot ID to be activated (is the way to encode
   PTRVAL pData1;       // the message -> activate one slot).
   PTRVAL pData2;       // 3 parameters for the object
   PTRVAL pData3;
};

// one light node information
struct tagLightInfo {
   float fAmbient;     // the ambient light level
   float fSource;      // the light level received from a light source
};



// USER STRINGS

// general messages
#define MSG_DEFAULT_ABORT_INFO "(problem in '%s', line %d)\n"
#define MSG_USER_ABORT "\nProgram aborted by user.\n"
#define MSG_WARN_END_REMARK "\nWarning: an end remark simbol found without start remark simbol.\n"
#define MSG_TOKEN_FOUND "Found: `%s' in file '%s'.\n"
#define MSG_STR_FILE_NAME "file name"
#define MSG_STR_ID "identifier"
#define MSG_READING_TEXTURE "Reading texture from PCX file '%s'.\n"
#define MSG_READING_LIGHTTABLE "Reading light table from LTB file '%s'.\n"
#define MSG_READING_TXE "\nReading TXE file '%s'.\n"
#define MSG_END_READING_TXE "End of reading TXE file '%s'.\n"
#define MSG_READING_PALETTE "Reading palette from PCX file '%s'.\n"
#define MSG_CONTINUE_READING_TXE "\nContinue reading TXE file '%s'...\n"
#define MSG_STR_OBJECTS "scenery objects"
#define MSG_STR_VALUE "numeric value"
#define MSG_FRAMES "\nTime: %f secs.\nFrames: %u.\nFrames per sec: %f\n"
#define MSG_DEFINE_ID "Define ID %d.\n"
#define MSG_SECTOR_ID "Sector ID %d.\n"

// component errors
#define MSG_ERROR_NO_CAST "\nERROR: not expecting the given cast.\n"
#define MSG_ERROR_NO_TYPE "\nERROR: component with the needed type not found.\n"

// scenery/mesh/engine errors
#define MSG_ERROR_CREATING_GR "\nError when alocating MCGA graphics class instance.\n"
#define MSG_ERROR_PALETTE_ALREADY_DEFINED "ERROR: palette already defined.\n"
#define MSG_ERROR_TEXTURE_ID_ALREADY_DEFINED "ERROR: texture ID `%d' already defined.\n"
#define MSG_ERROR_OBJECT_ID_ALREADY_DEFINED "ERROR: object instance ID `%d' already defined.\n"
#define MSG_ERROR_DEFINE_ID_ALREADY_DEFINED "ERROR: define ID `%d' already defined.\n"
#define MSG_ERROR_FUNCTION_ID_ALREADY_DEFINED "ERROR: function with ID `%d' already defined.\n"
#define MSG_ERROR_DEFINE_NOT_FOUND "ERROR: define '%d' not found when making object instanciation.\n"
#define MSG_ERROR_SECTOR_ID_ALREADY_DEFINED "ERROR: in define with ID `%d' the sector ID `%d' already defined.\n"
#define MSG_ERROR_LIGHTTABLE_ID_ALREADY_DEFINED "ERROR: light table ID `%d' already defined.\n"
#define MSG_ERROR_NO_LIGHTTABLE "ERROR: no light table defined with this ID.\n"
#define MSG_ERROR_NOPALETTE_FOR_LIGHTTABLE "ERROR: light table file not found and impossible create light table. No palette defined, load palette first.\n"
#define MSG_ERROR_INDEX_OUT_OF_RANGE "ERROR: index out of range, try correct declaring all polygons first and then their attributes.\n"
#define MSG_ERROR_ID_NOT_FOUND "ERROR: undefined ID.\n"
#define MSG_ERROR_NO_TEXTURE "ERROR: texture ID not found.\n"
#define MSG_ERROR_SECTOR_NOT_FOUND "ERROR: sector not found.\n"
#define MSG_ERROR_OBJECT_NOT_FOUND "ERROR: object not found.\n"
#define MSG_ERROR_NO_CAMERA "ERROR: camera object not defined.\n"
#define MSG_ERROR_DESTIN_OBJECT_NOT_FOUND "ERROR: destin object '%d' not found.\n"
#define MSG_ERROR_DESTIN_SECTOR_NOT_FOUND "ERROR: destin sector '%d' not found.\n"
#define MSG_ERROR_CAMERA_OBJECTID_NOT_FOUND "ERROR: object `%d' not found, changing current camera.\n"

// rendering errors
#define MSG_ERROR_LOW_VERTEXES "ERROR: not enough vertexes in polygon (minimum 3).\n"
#define MSG_ERROR_MANY_VERTEXES "ERROR: to much vertexes in polygon.\n"
#define MSG_ERROR_OVER_RESY_LIMIT "ERROR: to much resolution in Y frame buffer.\n"
#define MSG_ERROR_LOW_VERTEXES_IN_MESH "ERROR: not enough vertexes defined in mesh (minimum 3).\n"

// file/TXE errors
#define MSG_ERROR_NOBLOCK "\nERROR: no block found `%s'in `%s' block, BUGS?!\n"
#define MSG_ERROR_NOT_FOUND "ERROR: not found '%s'.\n"
#define MSG_ERROR_FILE_NOT_FOUND "\nERROR: file '%s' not found or error reading.\n", (char*) oszFile
#define MSG_ERROR_BLOCK "ERROR: unknow block `%s' found in `%s' block.\n"
#define MSG_ERROR_READING_TOKEN "\nERROR reading token, possible end of file found.\n"
#define MSG_ERROR_READING_FILE "\nERROR: reading file '%s'.\n", (char*) m_oszFile
#define MSG_ERROR_REMARK_OPENED "\nERROR: reading file '%s', remark still open in level %d.\n", (char*) m_oszFile, m_nRemarkLevel
#define MSG_ERROR_SPECTED "ERROR: not found %s after `%s'.\n"
#define MSG_ERROR_SPECTED_IN "ERROR: spected `%s' in `%s' block.\n"
#define MSG_ERROR_UNSPECTED_EOF "ERROR: unspected end-of-file found in file '%s', block type `%s' still open.\n"
#define MSG_ERROR_UNSPECTED_EOB "ERROR: unspected close block found in file '%s', block `%s'.\n"
#define MSG_ERROR_READING_BLOCK "Error reading block `%s' identified by `%d'.\n"

// memory errors
#define MSG_ERROR_MEMORY_LOW "\nERROR: impossible alocate memory, free memory and try again.\n"

// lighting errors
#define MSG_ERROR_INCORRECT_VERSION "\nERROR: incompatible light table, version not suported, try rebuild it.\n"

// backpropagation neural network errors
#define MSG_ERROR_TRAINING_VECTORS_UNDEFINIED "ERROR: define first the training vector.\n"

// sockets errors
#define MSG_ERROR_SOCKET_CREATE "ERROR: impossible create socket.\n"
#define MSG_ERROR_SOCKET_BIND_NAME "ERROR: impossible bind a name to socket.\n"
#define MSG_ERROR_CREATING_SOCKET "RROR: impossible create socket.\n"
#define MSG_ERROR_SOCKET_CREATION "ERROR: impossible create socket.\n"



// FILE SYSTEM

// reserved simbols
#define STR_READ "Read"
#define STR_TEXTURES "Textures"
#define STR_PALETTE "Palette"
#define STR_DEFINE "Define"
#define STR_SECTOR "Sector"
#define STR_POLYGONS "Polygons"
#define STR_VERTEXES "Vertexes"
#define STR_PORTALS "Portals"
#define STR_SOLIDSTATE "SolidState"
#define STR_ERROR "unknow-block-error"
#define STR_NOBLOCK "top-outside-all-blocks"
#define STR_DATA "simple data"
#define STR_OPENBLOCK "{"
#define STR_CLOSEBLOCK "}"
#define STR_OPENTUPLE "("
#define STR_CLOSETUPLE ")"
#define STR_OBJECTS "Objects"
#define STR_START "Start"
#define STR_LIGHTTABLES "LightTables"
#define STR_POLYGONSLIGHT "PolygonsLight"
#define STR_POLYGONSSURFACE "Surface"
#define STR_CMD_CREATE_COMPONENT "CreateComponent"
#define STR_CMD_ACTIVATE_COMPONENT "ActivateComponent"
#define STR_CMD_DESTROY_COMPONENT "DestroyComponent"
#define STR_INITMESSAGES "InitMessages"
#define STR_SETLIGHTLEVEL "SetLightLevel"
#define STR_SETLIGHTTABLE "SetLightTable"

#define STR_TUPLESEPARATOR ";"
#define STR_CONTINUATOR ".."

// id codes (used to define blocks types)
// ID_NOBLOCK is the topmost and ID_ERROR is a block not reconized (error)
// ID_DATA is a data found in the right place (no reserved word)

#define NUM_IDS 27

#define ID_ERROR 0
#define ID_NOBLOCK 1
#define ID_DATA 2
#define ID_READ 3
#define ID_TEXTURES 4
#define ID_PALETTE 5
#define ID_DEFINE 6
#define ID_SECTOR 7
#define ID_POLYGONS 8
#define ID_VERTEXES 9
#define ID_PORTALS 10
#define ID_SOLIDSTATE 11
#define ID_OPENBLOCK 12
#define ID_CLOSEBLOCK 13
#define ID_OPENTUPLE 14
#define ID_CLOSETUPLE 15
#define ID_OBJECTS 16
#define ID_START 17
#define ID_LIGHTTABLES 18
#define ID_POLYGONSLIGHT 19
#define ID_POLYGONSSURFACE 20
#define ID_CMD_CREATE_COMPONENT 21
#define ID_CMD_ACTIVATE_COMPONENT 22
#define ID_CMD_DESTROY_COMPONENT 23
#define ID_INITMESSAGES 24
#define ID_SETLIGHTLEVEL 25
#define ID_SETLIGHTTABLE 26



// SYSTEM SPECIFIC

// path and file name separator '/' or '\'
#define _3DFC_FILESEPARATOR '/'
#ifdef TRG_DOSDJGPP
#define _SYSTEM_FILESEPARATOR '\\'
#endif
#ifdef TRG_LINUXSVGA
#define _SYSTEM_FILESEPARATOR '/'
#endif
#ifdef TRG_LINUXXWIN
#define _SYSTEM_FILESEPARATOR '/'
#endif

// drive separator
#define DRIVESEPARATOR ':'



// VIDEO

// modes

#define MODE_TEXT 0
#define MODE_MCGA 1

// graphics output sizes

// video memory size needed for screen
#define VIDEOSIZE 64000
// VIDEOSIZE / 4
#define VIDEOSIZE4 16000

// resolution in graphics mode
#define RESX 320
#define RESY 200

// maximum Y resolution for frame buffers
#define MAX_RESY 256



// REFERENCE SYSTEM AND ENGINE

// Limits:

// how many units perform one round
#define ROUND 256


// Metrics and scales:

// pi
#ifndef PI
#define PI 3.14159265359
#endif

// Z clip plan position
#define Z_CLIP 80.0

// distance between projection plan and projection center
#define DIST_CP 230 

// camera radius
#define DEFAULT_RADIUS 50

// maximum speed
#define DEFAULT_MAXIMUM_SPEED 9

// scale factor for vertexes (position is position * SCALE)
#define DEFAULT_SCALE_FACTOR 1

// rendering scale factor (view data are scaled to * SCALE)
#define RENDERING_SCALE_FACTOR 8


// Lights and colors:

// signature for light table file
#define LIGHT_TABLE_SIGN "LT"

// version control for light table file
#define LIGHT_TABLE_VERSION 1

// number of palette colors
#define MAX_COLORS 256

// expoent of number of palette colors (NUM_COLORS = 2^NUM_COLORS_EXP)
#define MAX_COLORS_EXP 8

// palette range for each component: r, g, b (can be 0..63)
#define PAL_RANGE 64

// default light table
#define DEFAULT_LIGHT_TABLE 0

// default light level
#ifndef COMP_HILIGHTS
#define DEFAULT_LIGHT_LEVEL 1
#endif
#ifdef COMP_HILIGHTS
#define DEFAULT_LIGHT_LEVEL 0.5
#endif


// Texture:

// changing this values need review in CDraw horizontal line functions

// texture size
#define TEXTURE_SIZE 256

// texture max dimension (TEXTURE_SIZE-1)
#define TEXTURE_MAXDIM 255



// GERENAL USE MACROS

// key scan codes
#include "port/keycodes.h"

// verify if files header needed have been already included, if no include

#ifdef TRG_DOSDJGPP

#ifndef __dj_include_stdio_h_
#include <stdio.h>
#endif

#ifndef __dj_include_conio_h_
#include <conio.h>
#endif

#ifndef __dj_include_stdlib_h_
#include <stdlib.h>
#endif

#ifndef __dj_include_dos_h_
#include <dos.h>
#endif

#include "port/dosdjgpp/keyboard.h"

#endif

#include <stdlib.h>
#include <stdio.h>

#ifdef TRG_LINUXSVGA
#include <vga.h>     // vga graphics library for Linux   :)
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#endif

#ifdef TRG_LINUXXWIN
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#define XK_MISCELLANY 1
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysymdef.h>

#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Box.h>

#ifdef COMP_SHAREMEMORY
#include <X11/extensions/XShm.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif

#include "port/linxwin/keyboard.h"

#endif

// Abort program execution:
// set text mode, write message and terminate
#ifdef TRG_DOSDJGPP
   #define ABORT(MSG,FILE,LINE) \
   { \
      REGS \
         stRegsE, stRegsS; \
      stRegsE.x.ax = 0x0003; \
      int86 ( 0x10, &stRegsE, &stRegsS ); \
      printf ( MSG ); \
      printf ( MSG_DEFAULT_ABORT_INFO, FILE, LINE ); \
      exit ( 1 ); \
   }
#endif
#ifdef TRG_LINUXSVGA
   #define ABORT(MSG,FILE,LINE) \
   { \
      vga_setmode ( TEXT ); \
      printf ( MSG ); \
      printf ( MSG_DEFAULT_ABORT_INFO, FILE, LINE ); \
      exit ( 1 ); \
   }
#endif
#ifdef TRG_LINUXXWIN
   #define ABORT(MSG,FILE,LINE) \
   { \
      printf ( MSG ); \
      printf ( MSG_DEFAULT_ABORT_INFO, FILE, LINE ); \
      exit ( 1 ); \
   }
#endif

// Set pixel directily on video memory (only work in MCGA mode)
#ifdef TRG_DOSDJGPP
   #define SETPIXEL(X,Y,C) \
   { \
      BYTE \
         yCol = C; \
      if ( ( X ) >= 0 && ( X ) < 320 && ( Y ) >= 0 && ( Y ) < 200 ) \
         dosmemput ( (void*) &yCol, 1, 0xA0000 + ( X ) + ( ( Y ) * 320 ) ); \
   }
#endif

#ifdef TRG_DOSDJGPP
   // some debug macros
   #include "debug.h"
#endif



// end declarations
#endif
